(ns com.aurellem.run.final-cut
  (:use (com.aurellem.gb saves gb-driver util constants
                         items vbm characters money
                         rlm-assembly))
  (:use (com.aurellem.run util sound music title save-corruption
                          bootstrap-0 bootstrap-1
                          ram-display
                          ))
  (:require clojure.string)
  (:import [com.aurellem.gb.gb_driver SaveState])
  (:import java.awt.image.BufferedImage)
  (:import java.io.File)
  (:import com.aurellem.gb.WaveWriter))


(def render-dir (File. user-home "proj/vba-clojure/render"))

(defn render-files!
  ([^File target-dir initial-state moves]
     (let [ram-map-dir (File. target-dir "ram-map")
           frames-dir (File. target-dir "frames")
           audio-file (File. target-dir "audio.wav")
           _ (.mkdirs ram-map-dir)
           _ (.mkdirs frames-dir)
           wave-writer (WaveWriter. audio-file)
           moves (vec moves)]
       (set-state! initial-state)
       (dorun
        (map
         (fn [index]
           (run-moves @current-state (vector (moves index)))

           ;; record audio
           (.process wave-writer (sound-bytes)
                     gb-sound-format)

           ;; write screenshot
           (write-png!
            @current-state
            (File. frames-dir (format "%07d.png" index)))

           ;; write ram-image
           (write-ram-image!
            @current-state
            (File. ram-map-dir (format "%07d.png" index))))
         (range (count moves))))
       (.cleanup wave-writer)))
  ([initial-state moves]
     (render-files! render-dir initial-state moves))
  ([moves]
     (render-files! (root) moves)))

(defn file-names [#^File dir]
   (mapv #(.getCanonicalPath %) (next (sort (file-seq dir)))))

(defn composite-frames-command
  [screenshot ram-map target move]
  (let [actual-keys (set move)
        keys [:a :b :start :select :u :d :l :r]
        button-on-name  ["-size" "5x5" "xc:red"]
        button-off-name ["-size" "5x5" "xc:black"]]
    ["convert"
     "-size" "318x276" "xc:white"
     (map
      (fn [index]
        (let [position ["-geometry"
                        (str "+" (+ 30 (* index 8))
                             "+" "225") "-composite"]]
          (if (actual-keys (keys index))
            [button-on-name  position]
            [button-off-name position])))
      (range (count keys)))
     screenshot   "-geometry" "+10+10"  "-composite"
     ram-map      "-geometry" "+180+10" "-composite"
     target]))

(defn generate-composite-frames!
  [^File rendered-dir moves]
  (let [final-frames (File. rendered-dir "final")
        _ (.mkdir final-frames)
        screenshots (file-names (File. rendered-dir "frames"))
        ram-maps    (file-names (File. rendered-dir "ram-map"))
        targets (map #(.getCanonicalPath
                       (File. final-frames (format "%07d.bmp" %)))
                     (range (count screenshots)))]
    (dorun
     (map
      (comp
       (partial apply clojure.java.shell/sh)
       flatten
       (partial apply composite-frames-command)
       (fn [a b c d] (println c) [a b c d]))
      screenshots ram-maps targets moves))))


(defn final-cut! [^File render-dir]
  (let [movie (File. render-dir "final.ogg")
        final (File. render-dir "final")]
    (.delete movie)
    (clojure.java.shell/sh
     "ffmpeg" "-r" "60"
     "-i" (str (.getCanonicalPath final) "/" "%07d.bmp")
     "-i" (.getCanonicalPath (File. render-dir "audio.wav"))
     "-b:a" "128k"
     "-b:v" "9000k"
     "-c:a" "libvorbis"
     "-c:v" "libtheora"
     (.getCanonicalPath movie)) nil))
     
  
(comment

  ;; step 1
  
  (render-files!
   render-dir (root)
   (take 9000 (first (control-checkpoint))))

  ;; step 2
  
  (generate-composite-frames!
   render-dir 
   (take 9000 (first (control-checkpoint))))

  ;; step 3
  (final-cut! render-dir)
  )